/* --------------------------------------------------------------------------
PCSC-Lockin Copyright (c) 2011, EPFL LESO-PB, research group nanotechnology
for solar energy conversion, Mario Geiger André Kostro Andreas Schüler
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
   This product includes software developed by Mario Geiger, André Kostro
   and Andreas Schüler in LESO-PB lab, research group nanotechnology for
   solar energy conversion, at EPFL.
4. Neither the name of the EPFL LESO-PB nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY EPFL LESO-PB ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL EPFL LESO-PB BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

DESCRIPTION:

---------------------------------------------------------------------------- */

#ifndef VUMETER_H
#define VUMETER_H

#include <QtGui/QWidget>
#include <QtOpenGL>
#include <QMutex>

#include "mstream.h"

/*

 Show signals using openGL

 */

struct Channel {
    MStreamReader<float> *str;
    QColor color;
    float *circbuffer;
    int pos;
    bool full;
    float min;
    float max;
};

class Vumeter : public QGLWidget
{
    Q_OBJECT

public:
    Vumeter(int lenght = 500, bool globalrange = false, QWidget *parent = 0);
    Vumeter(QWidget *parent);
    ~Vumeter();

    void addStream(MStreamReader<float> *str, QColor color = Qt::white);
    void fixMinimum(float min);
    void fixMaximum(float max);

    void setLenght(int lenght);
    void setGlobalRange(bool global);

private:
    void initializeGL();
    void resizeGL(int w, int h);
    void paintGL();
    inline float yval(float y, float min, float max, float pixelmin, float pixelmax);
    void drawChannel(int i, float cmin, float cmax, float pmin, float pmax);

    void timerEvent(QTimerEvent *);
    void keyPressEvent(QKeyEvent *);
    void mousePressEvent(QMouseEvent *);
    void mouseDoubleClickEvent(QMouseEvent *);
    void wheelEvent(QWheelEvent *);

    QList<struct Channel> channels;

    int lenght;
    bool pause;

    bool globalrange;
    bool fixmin;
    float gmin;
    bool fixmax;
    float gmax;

    QMutex mutex;
};

#endif // VUMETER_H
